
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.006.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Quickly and easily create secure passwords",
  "dist_author" => [
    "Sawyer X <xsawyerx\@cpan.org>"
  ],
  "dist_name" => "App-Genpass",
  "dist_version" => "2.400",
  "license" => "perl",
  "module_name" => "App::Genpass",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Config::Any" => 0,
    "File::HomeDir" => 0,
    "File::Spec" => 0,
    "Getopt::Long" => 0,
    "List::AllUtils" => 0,
    "Math::Random::Secure" => "0.06",
    "Moo" => 0,
    "MooX::Types::MooseLike" => 0,
    "Sub::Quote" => 0,
    "namespace::clean" => "0.2",
    "strict" => 0,
    "warnings" => 0
  },
  "script_files" => [
    "bin/genpass"
  ],
  "test_requires" => {
    "File::Spec" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "List::MoreUtils" => 0,
    "Test::Deep" => 0,
    "Test::More" => 0,
    "blib" => "1.01",
    "perl" => "5.006"
  }
);


my %fallback_build_requires = (
  "File::Spec" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "List::MoreUtils" => 0,
  "Module::Build" => "0.28",
  "Test::Deep" => 0,
  "Test::More" => 0,
  "blib" => "1.01",
  "perl" => "5.006"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
